
#import "../libs/AppleTV.js"

// --------------------------------------------------------------------------------
//      Object: espn
//              trailers is an ATVAApp object for interacting with the
//              ESPN appliance on Apple TV.
// --------------------------------------------------------------------------------

if (typeof espn == 'undefined') {
    espn = new ATVAApp("ESPN");
}

if (typeof espn != 'undefined') {
    espn._objectName = "espn";

    // overriding fromMenu(), fromMenuAndHighlight(), goTo(), and goToAndHighlight() methods from base ATVAApp()

    espn.appMenu = function appMenu() {
        if ( ! this._appMenu ) {
            this._appMenu = new ATVAPageStore(this._title);
        }
        return( this._appMenu );
    }

    espn.fromMenu = function fromMenu() {
        if ( ! this.appMenu().waitForPage()) {
            UIALogger.logError(this._objectName + ".fromMenu(): failed to waiting for " + this._title);
            return(undefined);
        }
        return(this.appMenu());
    }

    espn.fromMenuAndHighlight = function fromMenuAndHighlight(name) {
        var menu = this.fromMenu();
        if (menu) {
            if (menu.moveToTabWithName(name)) {
                // force delay because UI might hangup
                atvaTarget.delay(10);
                return(menu);
            }
            UIALogger.logError(this._objectName + ".fromMenuAndHighlight(): failed to move to " + name + " on " + this._title);
            return(undefined);
        }
        return(undefined);
    }

    espn.fromMenuAndSelect = function fromMenuAndSelect(name) {
        var menu = this.fromMenuAndHighlight(name);
        if (menu) {
            menu.selectIntoPage();
            return(true);
        }
        UIALogger.logError(this._objectName + ".fromMenuAndSelect(): failed to select " + name + " on " + this._title);
        return(false);
    }

    espn.goTo = function goTo() {
        if (ATVAPageMainMenu.goToAndSelect(this._title)) {
            return(this.fromMenu());
        }
        UIALogger.logError(this._objectName + ".goTo(): failed to get to " + this._title);
        return(undefined);
    }

    espn.goToAndHighlight = function goToAndHighlight(name) {
        if ( ! this.goTo()) {
            return(undefined);
        }
        return(this.fromMenuAndHighlight(name));
    }

    espn.goToAndSelect = function goToAndSelect(name) {
        if ( ! this.goTo()) {
            return(undefined);
        }
        return(this.fromMenuAndSelect(name));
    }

    espn.isPlaybackError = function isPlaybackError() {
        this.appMenu().dismissScreenSaver();

        var theElements = appletv.mainWindow().elements().toArray();
        if ( ! theElements) {
            UIALogger.logError("No elements on screen to check");
        }
        else if (theElements.length <= 2) {
            var theName = undefined;
            for (var i = 0; i < theElements.length; ++i) {
                if (theElements[i].attributes().class == "UIAElement") {
                    theName = theElements[i].name();
                }
                else if (theElements[i].attributes().class == "UIAImage") {
                    // ignore
                }
                else {
                    // don't know this pattern
                    UIALogger.logError("Don't know this pattern.");
                    appletv.mainWindow().logElementTree();
                    return(false);
                }
            }

            if (theName && (theName == "Screen Saver")) {
                UIALogger.logError("Screen Saver is running on top of video playback.");
                return(false);
            }
            else if (theName && (theName == "Loading…")) {
                // Loading… isn't an error
                return(false);
            }
            else {
                if ( ! theName) {
                    theName = "Unknown error is displayed."
                }
                UIALogger.logError("Video playback has stopped unexpectly.");
                UIALogger.logError(theName);
                return(true);
            }
        }
        else if (ATVAPageMainMenu.isMainMenuDisplayed()) {
            UIALogger.logError("Popped back to main menu.");
            return(true);
        }

        return(false);
    }

    espn.isPlaybackStalled = function isPlaybackStalled() {
        espn.appMenu().dismissScreenSaver();

        if (appletv.mainWindow().elements()['Loading…'].isVisible() == 1) {
            // stalled
            return(true);
        }
        return(false);
    }

    espn.monitorPlayback = function monitorPlayback(interval) {
        // defaults
        if ( ! interval) {
            interval = 5 * 60; // 5 minutes
        }

        UIALogger.logMessage("Monitoring playback");
        while (true) {
            espn.appMenu().dismissScreenSaver();

            // check for error
            if (this.isPlaybackError()) {
                UIALogger.logError(Date() + ": playback has stopped");
                return(false);
            }
            if (this.isPlaybackStalled()) {
                UIALogger.logError(Date() + ": stalled");
            }
            atvaTarget.delay(interval);
        }
        return(false);
    }
}
