
// Don't import directly, import AppleTV.js

// ATVAPageAudioDetail.js handles detail pages on the AppleTV.

function ATVAPageAudioDetail(title) {
    ATVAPage.call(this, title);

    this.isPageDisplayed = function isPageDisplayed() {
        if (this.buttons()["Preview"].isValid()) {
            return(true);
        }

        return(false);
    }

    this.playPreview = function playPreview() {
        this.dismissScreenSaver();

        //UIALogger.logMessage("Playing Preview for " + this._title);
        if (this.moveToItemWithNameAndSelect("Preview")) {
            UIALogger.logMessage("AUDIO PLAYBACK SELECTED: " + Date());

            atvaTarget.delay(5);
            //var isPlayingScreen = new ATVAPage("Playing");
            //isPlayingScreen.isPageDisplayed = ATVAPageVideoDetail.isVideoPlaying;

            //if ( ! isPlayingScreen.waitForPage()) {
            //    UIALogger.logError("Never saw the video playing");
            //    return(false);
            //}
            //while (isPlayingScreen.elements()['Loading…'].isVisible() == 1) {
            //    // On video player, but loading.
            //    atvaTarget.delay(1);
            //}

            //UIALogger.logMessage("VIDEO PLAYBACK STARTED: " + Date());

            return(true);
        }
        return(false);
    }

    this.playItemName = function playItemName() {
        if (this._videoType == ATVAPageVideoDetail.NETFLIXTYPE) {
            return("Play " + this._title);
        }
        else if (this._videoType == ATVAPageVideoDetail.VIMEOTYPE) {
            return("Play Video");
        }
        else if (this._videoType == ATVAPageVideoDetail.TRAILERSTYPE) {
            return("Play, hdContent");
        }
        else {
            return("Play");
        }
    }

    this.play = function play(shouldResume) {
        this.dismissScreenSaver();
        var itemName = this.playItemName();

        //UIALogger.logMessage("Playing for " + this._title);
        if (this.moveToItemWithNameAndSelect(itemName)) {
            UIALogger.logMessage("PLAYBACK SELECTED: " + Date());

            var resumeMenu = new ATVAPage("Resume");
            resumeMenu.isPageDisplayed = ATVAPageVideoDetail.isResumeMenu;

            var isPlayingScreen = new ATVAPage("Playing");
            if (this._videoType == ATVAPageVideoDetail.PODCASTAUDIOTYPE) {
                isPlayingScreen.isPageDisplayed = ATVAPageVideoDetail.isAudioPodcastPlaying;
            }
            else {
                isPlayingScreen.isPageDisplayed = ATVAPageVideoDetail.isVideoPlaying;
            }

            var isErrorScreen = new ATVAPage("Error");
            isErrorScreen.isPageDisplayed = ATVAPageVideoDetail.isPlaybackError;

            var readyToPlayMenu = undefined;
            var pages = undefined;

            if (this._videoType == ATVAPageVideoDetail.MOVIETYPE) {
                readyToPlayMenu = new ATVAPage("Ready to Play");
                readyToPlayMenu.isPageDisplayed = ATVAPageVideoDetail.isReadyToPlayMenu;

                pages = new Array(resumeMenu, readyToPlayMenu, isPlayingScreen, isErrorScreen);
            }
            else {
                pages = new Array(resumeMenu, isPlayingScreen, isErrorScreen);
            }

            // Loading and isPlayingScreen can look a like, put extra delay to get around
            atvaTarget.delay(10);

            UIALogger.logMessage("  waiting for resume");
            if ( ! ATVAPage.waitForOneOfThesePages(pages, 5, 60)) {
                return (false);
            }

            // One of the above pages is shown.

            // dismissScreenSaver is call in waitForPage, we need to call it manually below
            // incase the Screen Saver Fired.
            this.dismissScreenSaver();

            if (isErrorScreen.isPageDisplayed()) {
                return(false);
            }

            if (resumeMenu.isPageDisplayed()) {
                if ( ! shouldResume) {
                    UIALogger.logMessage("  selecting start from beginning");
                    if ( ! resumeMenu.moveToItemWithNameAndSelect("Start from Beginning")) {
                        return(false);
                    }
                }
                else {
                    UIALogger.logMessage("  selecting resume");
                    if (resumeMenu.moveToItemWithNameAndSelect("Resume Playing")) {
                        return(false);
                    }
                }

                // need to wait again for next screen
                if (this._videoType == ATVAPageVideoDetail.MOVIETYPE) {
                    pages = new Array(readyToPlayMenu, isPlayingScreen, isErrorScreen);
                }
                else {
                    pages = new Array(isPlayingScreen, isErrorScreen);
                }

                if ( ! ATVAPage.waitForOneOfThesePages(pages, 5, 60)) {
                    return (false);
                }
            }

            UIALogger.logMessage("  done with resume");

            // Ready to Play is displayed.
            this.dismissScreenSaver();

            if (isErrorScreen.isPageDisplayed()) {
                return(false);
            }

            // might be ready to play
            if (readyToPlayMenu && readyToPlayMenu.isPageDisplayed()) {
                // First the ReadyToPlayMenu might be displayed, but if the caching happens fast
                // enough, this will go away by itself.
                while (readyToPlayMenu.isPageDisplayed()) {
                    this.dismissScreenSaver();
                    if (ATVAPageVideoDetail.isReadyToPlayDisplayed(readyToPlayMenu)) {
                        // Notice that even after selecting this, it will still take time to authorize.
                        UIALogger.logMessage("VIDEO PLAYBACK SELECTING READY TO PLAY: " + Date());
                        readyToPlayMenu.selectIntoPage();

                        break; // while is done
                    }
                    else {
                        UIALogger.logMessage("VIDEO PLAYBACK CACHING: " + Date());
                        atvaTarget.delay(10);
                    }

                    // Incase a long wait.
                    this.dismissScreenSaver();
                }
                UIALogger.logMessage("VIDEO PLAYBACK CACHING FINISHED: " + Date());
            }

            // need to wait again
            pages = new Array(isPlayingScreen, isErrorScreen);
            if ( ! ATVAPage.waitForOneOfThesePages(pages, 5, 60)) {
                return (false);
            }

            if (isErrorScreen.isPageDisplayed()) {
                return(false);
            }

            if ( ! isPlayingScreen.isPageDisplayed()) {
                UIALogger.logError("Never saw the video playing");
                return(false);
            }

            if (this._videoType != ATVAPageVideoDetail.PODCASTAUDIOTYPE) {
                // video player is now displayed, but might be loading.
                while (isPlayingScreen.elements()['Loading…'].isVisible() == 1) {
                    // On video player, but loading.
                    atvaTarget.delay(1);
                }
            }

            UIALogger.logMessage("PLAYBACK STARTED: " + Date());

            return(true);
        }
        return(false);
    }

    this.waitForEnd = function waitForEnd(interval, timeout, itemName) {
        if (typeof itemName == 'undefined') {
            itemName = this.playItemName();
        }

        // defaults
        if ( ! interval) {
            interval = 10; // 10 seconds
        }
        if ( ! timeout) {
            timeout = 10 * 60; // 5 minutes
        }

        UIALogger.logMessage("Waiting for end of " + this._title);
        for (var theTimer = 0; theTimer < timeout; theTimer += interval) {
            this.dismissScreenSaver();

            // check for error
            //if (ATVAPageVideoDetail.isPlaybackError()) {
            //    return(false);
            //}

            if (this.buttons()[itemName].isValid()) {
                // returned to details screen, playback is complete
                //UIALogger.logMessage("  end");
                return(true);
            }
            UIALogger.logMessage("  waiting " + interval + "s: " + Date());
            atvaTarget.delay(interval);
        }
        // failed to return to details screen. Might still be playing or might be somewhere else.
        return(false);
    }

    this.playPreviwAndWaitForEnd = function playPreviwAndWaitForEnd(interval, timeout) {
        if (this.playPreview()) {
            return(this.waitForEnd(interval, timeout, "Preview"));
        }
        return(false);
    }

    this.playAndWaitForEnd = function playAndWaitForEnd(shouldResume, interval, timeout) {
        if (this.play(shouldResume)) {
            return(this.waitForEnd(interval, timeout));
        }
        return(false);
    }

    return (this);
}
