
#import "Music.js"

/**
 *       Function: musicTest_goToMusic()
 *           Go to Main Menu>Music and highlight item.
 *   
 *  \param itemName (string) - item on main menu to highlight
 *  \param takeHeap (string) - (optional) true to collect process information
 *  \param processName (string) - (optional) process to take information
 *  \param infoFlags (string) - (optional) flag for information to take
 *   
 *  \return 1 if successfully signed in, otherwise 0
 **/

function musicTest_goToMusic() {
    if (this.argv.length < 1) {
        this.testError = "Item to highlight is required";
        return this.testResult = 0;
    }
    var itemName = this.argv[0];
    var takeHeap = false;
    var processName = undefined;
    var infoFlags = undefined;
    if (this.argv.length >= 3) {
        if (this.argv[1].toUpperCase() == "TRUE") {
            takeHeap = true;
        }
        processName = this.argv[2];
        if (this.argv.length >= 4) {
            infoFlags = this.argv[3];
        }
        else {
            infoFlags = "";
        }
    }

    if ( ! music.goToAndHighlight(itemName)) {
        this.testError = "Failed to highlight item";
        this.testResult = 0;
    }
    else {
        this.testResult = 1;
    }

    if (takeHeap) {
        // wait 5 seconds before taking heap
        atvaTarget.delay(5);
        var output = new ATVAOutput("/var/root/heaps");
        output.addInfomationToCollect(processName, infoFlags);
        output.collectProcessInfomation("On Main Menu>Music>" + itemName);
    }
    return this.testResult;
}

/**
 *       Function: musicTest_browseTopMusicAndPlayPreview.
 *           Search for track through Movies>Top Music.
 *   
 *  \param row (string) - row of item to play
 *  \param column (string) - column of item to play
 *  \param track (string) - track number of item to play
 *   
 *  \return 1 if found, otherwise 0
 **/

function musicTest_browseTopMusicAndPlayPreview() {
    if (this.argv.length < 3) {
        this.testError = "Top movie row and column need to be given.";
        return this.testResult = 0;
    }
    var row = parseInt(this.argv[0]);
    var column = parseInt(this.argv[1]);
    var track = parseInt(this.argv[2]);

    var album = music.goToBrowseTopMusicRowColumn(row, column);

    if ( ! album) {
        this.testError = "Failed to get to row = " + row + " column = " + column;
        return this.testResult = 0;
    }

    var dp = music.fromAlbumSelectTrack(album.title(), track);

    if ( ! dp) {
        this.testError = "Fail to find track " + track + " at row = " + row + " column = " + column;
        return this.testResult = 0;
    }

    if ( ! dp.playPreviwAndWaitForEnd()) {
        this.testError = "Fail to play audio preview";
        return this.testResult = 0;
    }
    else {
        return this.testResult = 1;
    }
}
